@@ -1,12 +1,16 @@
 # This script handles the SIP configuration and generates the Makefiles.
 #
-# Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
-# 
+# Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+#
 # This file is part of SIP.
-# 
+#
 # This copy of SIP is licensed for use under the terms of the SIP License
 # Agreement.  See the file LICENSE for more details.
-# 
+#
+# This copy of SIP may also used under the terms of the GNU General Public
+# License v2 or v3 as published by the Free Software Foundation which can be
+# found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+#
 # SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
@@ -21,8 +25,8 @@ import siputils
 
 
 # Initialise the globals.
-sip_version = 0x040802
-sip_version_str = "4.8.2"
+sip_version = 0x040b02
+sip_version_str = "4.11.2"
 py_version = sys.hexversion >> 8
 plat_py_site_dir = None
 plat_py_inc_dir = None
@@ -31,9 +35,11 @@ plat_py_lib_dir = None
 plat_sip_dir = None
 plat_bin_dir = None
 platform_specs = []
+src_dir = os.path.dirname(os.path.abspath(__file__))
 
 # Constants.
-DEFAULT_MACOSX_SDK = 'MacOSX10.5.sdk'
+DEFAULT_MACOSX_ARCH = 'i386 ppc'
+MACOSX_SDK_DIR = '/Developer/SDKs'
 
 # Command line options.
 default_platform = None
@@ -68,18 +74,18 @@ build_macro_names = [
     "DEL_FILE",
     "EXTENSION_SHLIB", "EXTENSION_PLUGIN",
     "INCDIR", "INCDIR_X11", "INCDIR_OPENGL",
-    "LIBS_CORE", "LIBS_GUI", "LIBS_NETWORK", "LIBS_OPENGL",
+    "LIBS_CORE", "LIBS_GUI", "LIBS_NETWORK", "LIBS_OPENGL", "LIBS_WEBKIT",
     "LINK", "LINK_SHLIB", "AIX_SHLIB", "LINK_SHLIB_CMD",
     "LFLAGS", "LFLAGS_CONSOLE", "LFLAGS_CONSOLE_DLL", "LFLAGS_DEBUG",
     "LFLAGS_DLL",
     "LFLAGS_PLUGIN", "LFLAGS_RELEASE", "LFLAGS_SHLIB", "LFLAGS_SONAME",
     "LFLAGS_THREAD", "LFLAGS_WINDOWS", "LFLAGS_WINDOWS_DLL", "LFLAGS_OPENGL",
     "LIBDIR", "LIBDIR_X11", "LIBDIR_OPENGL",
-    "LIBS", "LIBS_CONSOLE", "LIBS_OPENGL", "LIBS_OPENGL", "LIBS_RT",
+    "LIBS", "LIBS_CONSOLE", "LIBS_RT",
     "LIBS_RTMT", "LIBS_THREAD", "LIBS_WINDOWS", "LIBS_X11",
     "MAKEFILE_GENERATOR",
     "MKDIR",
-    "RPATH",
+    "RPATH", "LFLAGS_RPATH",
     "AR", "RANLIB", "LIB", "STRIP"
 ]
 
@@ -173,6 +179,9 @@ def inform_user():
     siputils.inform("The default directory to install .sip files in is %s." % opts.sipsipdir)
     siputils.inform("The platform/compiler configuration is %s." % opts.platform)
 
+    if opts.arch:
+        siputils.inform("MacOS/X binaries will be created for %s." % (", ".join(opts.arch.split())))
+
     if opts.universal:
         siputils.inform("MacOS/X universal binaries will be created using %s." % opts.universal)
 
@@ -226,7 +235,8 @@ def create_config(module, template, macros):
         "py_inc_dir":       plat_py_inc_dir,
         "py_conf_inc_dir":  plat_py_conf_inc_dir,
         "py_lib_dir":       plat_py_lib_dir,
-        "universal":        opts.universal
+        "universal":        opts.universal,
+        "arch":             opts.arch
     }
 
     siputils.create_config_module(module, template, content, macros)
@@ -237,7 +247,8 @@ def create_makefiles(macros):
 
     macros is the dictionary of platform specific build macros.
     """
-    # Bootstrap.
+    # Bootstrap.  Make sure we get the right one.
+    sys.path.insert(0, os.path.curdir)
     import sipconfig
 
     cfg = sipconfig.Configuration()
@@ -249,34 +260,37 @@ def create_makefiles(macros):
     sipconfig.ParentMakefile(
         configuration=cfg,
         subdirs=["sipgen", "siplib"],
-        installs=(["sipconfig.py", "sipdistutils.py"], cfg.sip_mod_dir)
+        installs=(["sipconfig.py", os.path.join(src_dir, "sipdistutils.py")],
+                cfg.sip_mod_dir)
     ).generate()
 
     sipconfig.inform("Creating sip code generator Makefile...")
 
     sipconfig.ProgramMakefile(
         configuration=cfg,
-        build_file="sipgen.sbf",
+        build_file=os.path.join(src_dir, "sipgen", "sipgen.sbf"),
         dir="sipgen",
         install_dir=os.path.dirname(cfg.sip_bin),
         console=1,
         warnings=0,
-        universal=opts.universal
+        universal=opts.universal,
+        arch=opts.arch
     ).generate()
 
     sipconfig.inform("Creating sip module Makefile...")
 
     makefile = sipconfig.ModuleMakefile(
         configuration=cfg,
-        build_file="siplib.sbf",
+        build_file=os.path.join(src_dir, "siplib", "siplib.sbf"),
         dir="siplib",
         install_dir=cfg.sip_mod_dir,
-        installs=(["sip.h"], cfg.sip_inc_dir),
+        installs=([os.path.join(src_dir, "siplib", "sip.h")], cfg.sip_inc_dir),
         console=1,
         warnings=0,
         static=opts.static,
         debug=opts.debug,
-        universal=opts.universal
+        universal=opts.universal,
+        arch=opts.arch
     )
 
     makefile.generate()
@@ -294,20 +308,36 @@ def create_optparser():
     # Note: we don't use %default to be compatible with Python 2.3.
     p.add_option("-k", "--static", action="store_true", default=False,
             dest="static", help="build the SIP module as a static library")
-    p.add_option("-n", "--universal", action="store_true", default=False,
-            dest="universal", help="build the SIP code generator and module "
-            "as universal binaries on MacOS/X")
     p.add_option("-p", "--platform", action="store",
             default=default_platform, type="string", metavar="PLATFORM",
             dest="platform", help="the platform/compiler configuration "
             "[default: %s]" % default_platform)
-    p.add_option("-s", "--sdk", action="store",
-            default=DEFAULT_MACOSX_SDK, type="string", metavar="SDK",
-            dest="sdk", help="the name of the MacOS/X SDK used when building "
-            "universal binaries [default: %s]" % DEFAULT_MACOSX_SDK)
     p.add_option("-u", "--debug", action="store_true", default=False,
             help="build with debugging symbols")
 
+    if sys.platform == 'darwin':
+        # Get the latest SDK to use as the default.
+        sdks = glob.glob(MACOSX_SDK_DIR + '/MacOSX*.sdk')
+        if len(sdks) > 0:
+            sdks.sort()
+            _, default_sdk = os.path.split(sdks[-1])
+        else:
+            default_sdk = 'MacOSX10.4u.sdk'
+
+        g = optparse.OptionGroup(p, title="MacOS X Configuration")
+        g.add_option("--arch", action="append", default=[], dest="arch",
+                choices=["i386", "x86_64", "ppc"],
+                help="build for architecture ARCH")
+        g.add_option("-n", "--universal", action="store_true", default=False,
+                dest="universal",
+                help="build the SIP code generator and module as universal "
+                        "binaries")
+        g.add_option("-s", "--sdk", action="store", default=default_sdk,
+                type="string", metavar="SDK", dest="sdk",
+                help="the name of the SDK used when building universal "
+                        "binaries [default: %s]" % default_sdk)
+        p.add_option_group(g)
+
     # Querying.
     g = optparse.OptionGroup(p, title="Query")
     g.add_option("--show-platforms", action="store_true", default=False,
@@ -356,7 +386,7 @@ def main(argv):
     set_platform_directories()
 
     # Build up the list of valid specs.
-    for s in os.listdir("specs"):
+    for s in os.listdir(os.path.join(src_dir, "specs")):
         platform_specs.append(s)
 
     # Parse the command line.
@@ -366,6 +396,12 @@ def main(argv):
     p = create_optparser()
     opts, args = p.parse_args()
 
+    # Make sure MacOS specific options get initialised.
+    if sys.platform != 'darwin':
+        opts.universal = ''
+        opts.arch = []
+        opts.sdk = ''
+
     # Handle the query options.
     if opts.show_platforms or opts.show_build_macros:
         if opts.show_platforms:
@@ -376,21 +412,32 @@ def main(argv):
 
         sys.exit()
 
+    # Convert the list 'arch' option to a string.  Multiple architectures
+    # imply a universal binary.
+    if len(opts.arch) > 1:
+        opts.universal = True
+
+    opts.arch = ' '.join(opts.arch)
+
     # Convert the boolean 'universal' option to a string.
     if opts.universal:
         if '/' in opts.sdk:
             opts.universal = os.path.abspath(opts.sdk)
         else:
-            opts.universal = '/Developer/SDKs/' + opts.sdk
+            opts.universal = MACOSX_SDK_DIR + '/' + opts.sdk
 
         if not os.path.isdir(opts.universal):
-            siputils.error("Unable to find the SDK directory %s. Use the -s flag to specify the name of the SDK (e.g. %s) or its full path." % (opts.universal, DEFAULT_MACOSX_SDK))
+            siputils.error("Unable to find the SDK directory %s. Use the --sdk flag to specify the name of the SDK or its full path." % opts.universal)
+
+        if opts.arch == '':
+            opts.arch = DEFAULT_MACOSX_ARCH
     else:
         opts.universal = ''
 
     # Get the platform specific macros for building.
-    macros = siputils.parse_build_macros(os.path.join("specs", opts.platform),
-            build_macro_names, args)
+    macros = siputils.parse_build_macros(
+            os.path.join(src_dir, "specs", opts.platform), build_macro_names,
+            args)
 
     if macros is None:
         p.print_help()
@@ -400,7 +447,8 @@ def main(argv):
     inform_user()
 
     # Install the configuration module.
-    create_config("sipconfig.py", "siputils.py", macros)
+    create_config("sipconfig.py", os.path.join(src_dir, "siputils.py"),
+            macros)
 
     # Create the Makefiles.
     create_makefiles(macros)
